==> bgameb-0.0.16.tar.gz: bgameb-0.0.16/bgameb/stuff.py <==
"""Game dices, coins, cards and other stuffs
"""
import random
from abc import ABC
from typing import List, Optional, Literal
from collections import Counter
from dataclasses import dataclass, field
from dataclasses_json import config
from bgameb.base import Base
from bgameb.errors import StuffDefineError


@dataclass
class BaseStuff(Base, ABC):
    """Base class for game stuff (like dices or cards)
    """


@dataclass
class RollerType(BaseStuff):
    """Base class for define types of rollers or fliped objects

    Define name to identify later this object by unique name.
    For example: 'six_side_dice'

    Sides attr define number of sides of roller. Default to 2.
    Sides can't be less than 2, because one-sided roller is
    strongly determined and 0zero-sided is imposible.

    .. code-block::
        :caption: Example:

            dice = RollerType(name='coin', sides=2)

    Raises:
        StuffDefineError: number of sides less than 2
    """
    sides: int = 2

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.sides < 2:
            raise StuffDefineError(
                message=f'Number of sides={self.sides} '
                        f'for {self.name}. Needed >= 2.',
                logger=self.logger
                )


@dataclass
class Roller(RollerType):
    """Creare the roller (like dice or coin)
    """
    _range: List[int] = field(
        default_factory=list,
        metadata=config(exclude=lambda x: True),
        init=False,
        repr=False
        )
    count: int = 0

    def __post_init__(self) -> None:
        super().__post_init__()
        self._range = list(range(1, self.sides + 1))

    def roll(self) -> List[int]:
        """Roll and return result

        Returns:
            List[int]: result of roll
        """
        roll = [
            random.choices(self._range, k=1)[0] for _
            in list(range(self.count))
            ]
        self.logger.debug(f'Is rolled {roll=}')
        return roll


@dataclass
class CardType(BaseStuff):
    """Create the card

    Define name to identify later this object by unique name.
    For example: 'unique_card'

    .. code-block::
        :caption: Example:

            card = CardType(name='unique_card')
    """
    open: bool = False
    tapped: bool = False
    side: Optional[str] = None
    count: int = 0

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass
class Card(CardType):
    """Create the card
    """
    counter: Counter = field(
        default_factory=Counter,
        init=False,
        )
    count: int = field(
        default=0,
        metadata=config(exclude=lambda x: True),
        repr=False
        )

    def __post_init__(self) -> None:
        super().__post_init__()

    def flip(self) -> None:
        """Face up or face down the card regardles of it condition
        """
        if self.open:
            self.open = False
            self.logger.debug(f'Card face down.')
        else:
            self.open = True
            self.logger.debug(f'Card face up.')

    def face_up(self) -> None:
        """Face up the card and return text
        """
        self.open = True
        self.logger.debug(f'Card face up.')

    def face_down(self) -> None:
        """Face down the card
        """
        self.open = False
        self.logger.debug(f'Card face down.')

    def tap(self, side='right') -> None:
        """Tap the card to the given side

        Args:
            side (str, optional): sifde to tap. Defaults to 'right'.
        """
        self.tapped = True
        self.side = side
        self.logger.debug(f'Card taped to side {side}.')

    def untap(self) -> None:
        """Untap the card
        """
        self.tapped = False
        self.logger.debug(f'Card untaped.')

    def alter(self) -> None:
        """Many cards have alter views. For example
        card can have main view, that apply most time of the game
        and second view, that apply only if card played as
        that alternative. For ease of understanding, consider that
        different views of the same card are not related directly
        to each other.
        """
        raise NotImplementedError


STUFF = {
    'roller': RollerType,
    'card': CardType,
}
STUFF_TYPES = Literal['roller', 'card']

==> bgameb-0.0.20.tar.gz: bgameb-0.0.20/bgameb/stuff.py <==
"""Game dices, coins, cards and other stuffs
"""
import random
from typing import List, Optional, Literal
from collections import Counter
from dataclasses import dataclass, field
from dataclasses_json import config
from bgameb.base import Base
from bgameb.errors import StuffDefineError


@dataclass
class BaseStuff(Base):
    """Base class for game stuff (like dices or cards)
    """
    count: int = 1


@dataclass
class Rule(BaseStuff):
    """Rule object
    """
    count: int = field(
        default=1,
        metadata=config(exclude=lambda x: True),  # type: ignore
        repr=False
        )
    text: Optional[str] = None
    type_: str = field(
        default='rule',
        metadata=config(exclude=lambda x: True),  # type: ignore
        repr=False
        )

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass
class Dice(BaseStuff):
    """Base class for define types of rolled or fliped objects

    Define name to identify later this object by unique name.
    For example: 'six_side'

    Sides attr define number of sides of roller. Default to 2.
    Sides can't be less than 2, because one-sided roller is
    strongly determined and 0zero-sided is imposible.

    .. code-block::
        :caption: Example:

            dice = Dice(name='coin', sides=2)

    Raises:
        StuffDefineError: number of sides less than 2
    """
    _range: List[int] = field(
        default_factory=list,
        metadata=config(exclude=lambda x: True),  # type: ignore
        init=False,
        repr=False
        )
    sides: int = 2
    type_: str = field(
        default='dice',
        metadata=config(exclude=lambda x: True),  # type: ignore
        repr=False
        )

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.sides < 2:
            raise StuffDefineError(
                message=f'Number of sides={self.sides} '
                        f'for {self.name}. Needed >= 2.',
                logger=self.logger
                )
        self._range = list(range(1, self.sides + 1))

    def roll(self) -> List[int]:
        """Roll and return result

        Returns:
            List[int]: result of roll
        """
        roll = [
            random.choices(self._range, k=1)[0] for _
            in list(range(self.count))
            ]
        self.logger.debug(f'Is rolled {roll=}')
        return roll


@dataclass
class Card(BaseStuff):
    """Create the card

    Define name to identify later this object by unique name.
    For example: 'unique_card'

    .. code-block::
        :caption: Example:

            card = CardType(name='unique_card')
    """
    open: bool = False
    tapped: bool = False
    side: Optional[str] = None
    counter: Counter = field(default_factory=Counter)
    type_: str = field(
        default='card',
        metadata=config(exclude=lambda x: True),  # type: ignore
        repr=False
        )

    def __post_init__(self) -> None:
        super().__post_init__()

    def flip(self) -> None:
        """Face up or face down the card regardles of it condition
        """
        if self.open:
            self.open = False
            self.logger.debug(f'Card face down.')
        else:
            self.open = True
            self.logger.debug(f'Card face up.')

    def face_up(self) -> None:
        """Face up the card and return text
        """
        self.open = True
        self.logger.debug(f'Card face up.')

    def face_down(self) -> None:
        """Face down the card
        """
        self.open = False
        self.logger.debug(f'Card face down.')

    def tap(self, side='right') -> None:
        """Tap the card to the given side

        Args:
            side (str, optional): sifde to tap. Defaults to 'right'.
        """
        self.tapped = True
        self.side = side
        self.logger.debug(f'Card taped to side {side}.')

    def untap(self) -> None:
        """Untap the card
        """
        self.tapped = False
        self.logger.debug(f'Card untaped.')

    def alter(self) -> None:
        """Many cards have alter views. For example
        card can have main view, that apply most time of the game
        and second view, that apply only if card played as
        that alternative. For ease of understanding, consider that
        different views of the same card are not related directly
        to each other.
        """
        raise NotImplementedError


STUFF = {
    Rule.type_: Rule,
    Dice.type_: Dice,
    Card.type_: Card,
    }
STUFF_TYPES = Literal['rule', 'dice', 'card']
